@@ -27,7 +27,25 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: pdirect.h,v $
+ * Revision 1.43  2005/11/25 03:43:47  csoutheren
+ * Fixed function argument comments to be compatible with Doxygen
+ *
+ * Revision 1.42  2004/09/17 03:51:15  csoutheren
+ * More fixes for PDirectory problem
+ *
+ * Revision 1.42  2004/09/15 05:11:39  csoutheren
+ * Fixed problem with PDirectory destructor not calling Close()
+ * Thanks to Paul Long
+ *
+ * Revision 1.41  2003/09/17 05:41:58  csoutheren
+ * Removed recursive includes
+ *
+ * Revision 1.40  2003/09/17 01:18:02  csoutheren
+ * Removed recursive include file system and removed all references
+ * to deprecated coooperative threading support
+ *
  * Revision 1.39  2003/06/06 15:04:17  dsandras
+ *
  * Fixed compilation warning with gcc 3.3 by removing the PINLINE
  *
  * Revision 1.38  2002/11/20 00:13:43  robertj
@@ -147,6 +165,7 @@
  */
 
 
+#ifndef _PDIRECTORY
 #define _PDIRECTORY
 
 #ifdef P_USE_PRAGMA
@@ -157,6 +176,16 @@
 #undef Fifo
 #endif
 
+#ifdef _WIN32
+#define PDIR_SEPARATOR '\\'
+const PINDEX P_MAX_PATH = _MAX_PATH;
+typedef PCaselessString PFilePathString;
+#else
+#define PDIR_SEPARATOR '/'
+#define P_MAX_PATH    (_POSIX_PATH_MAX)
+typedef PString PFilePathString;
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 // File System
 
@@ -298,7 +327,7 @@ class PDirectory : public PFilePathString
        made absolute by the creation of the #PDirectory# object.
      */
     PDirectory(
-      const char * cpathname      /// Directory path name for new object.
+      const char * cpathname      ///< Directory path name for new object.
     );
 
     /**Create a directory object of the specified directory. The
@@ -306,19 +335,19 @@ class PDirectory : public PFilePathString
        made absolute by the creation of the #PDirectory# object.
      */
     PDirectory(
-      const PString & pathname    /// Directory path name for new object.
+      const PString & pathname    ///< Directory path name for new object.
     );
 
     /**Set the directory to the specified path.
      */
     PDirectory & operator=(
-      const PString & pathname    /// Directory path name for new object.
+      const PString & pathname    ///< Directory path name for new object.
     );
 
     /**Set the directory to the specified path.
      */
     PDirectory & operator=(
-      const char * cpathname      /// Directory path name for new object.
+      const char * cpathname      ///< Directory path name for new object.
     );
   //@}
 
@@ -372,7 +401,7 @@ class PDirectory : public PFilePathString
        TRUE if may be used to separate directories in a path.
      */
     PINLINE static BOOL IsSeparator(
-      char ch    /// Character to check as being a separator.
+      char ch    ///< Character to check as being a separator.
     );
 
     /**Determine the total number of bytes and number of bytes free on the
@@ -386,9 +415,9 @@ class PDirectory : public PFilePathString
        TRUE if the information could be determined.
      */
     BOOL GetVolumeSpace(
-      PInt64 & total,     /// Total number of bytes available on volume
-      PInt64 & free,      /// Number of bytes unused on the volume
-      DWORD & clusterSize /// "Quantisation factor" in bytes for files on volume
+      PInt64 & total,     ///< Total number of bytes available on volume
+      PInt64 & free,      ///< Number of bytes unused on the volume
+      DWORD & clusterSize ///< "Quantisation factor" in bytes for files on volume
     ) const;
   //@}
 
@@ -407,7 +436,7 @@ class PDirectory : public PFilePathString
        TRUE if directory exists.
      */
     static BOOL Exists(
-      const PString & path   /// Directory file path.
+      const PString & path   ///< Directory file path.
     );
       
     /**Change the current working directory to the objects location.
@@ -423,7 +452,7 @@ class PDirectory : public PFilePathString
        TRUE if current working directory was changed.
      */
     static BOOL Change(
-      const PString & path   /// Directory file path.
+      const PString & path   ///< Directory file path.
     );
       
     /**Create a new directory with the specified permissions.
@@ -440,8 +469,8 @@ class PDirectory : public PFilePathString
        TRUE if directory created.
      */
     static BOOL Create(
-      const PString & p,   /// Directory file path.
-      int perm = PFileInfo::DefaultDirPerms    /// Permission on new directory.
+      const PString & p,   ///< Directory file path.
+      int perm = PFileInfo::DefaultDirPerms    ///< Permission on new directory.
     );
 
     /**Delete the directory.
@@ -457,7 +486,7 @@ class PDirectory : public PFilePathString
        TRUE if directory was deleted.
      */
     static BOOL Remove(
-      const PString & path   /// Directory file path.
+      const PString & path   ///< Directory file path.
     );
   //@}
 
@@ -478,8 +507,8 @@ class PDirectory : public PFilePathString
        TRUE if directory was successfully opened, and there was at least one
        file in it of the specified types.
      */
-    BOOL Open(
-      int scanMask = PFileInfo::AllFiles    /// Mask of files to provide.
+    virtual BOOL Open(
+      int scanMask = PFileInfo::AllFiles    ///< Mask of files to provide.
     );
       
     /**Restart file list scan from the beginning of directory. This is similar
@@ -496,8 +525,8 @@ class PDirectory : public PFilePathString
        TRUE if directory was successfully opened, and there was at least one
        file in it of the specified types.
      */
-    BOOL Restart(
-      int scanMask = PFileInfo::AllFiles    /// Mask of files to provide.
+    virtual BOOL Restart(
+      int scanMask = PFileInfo::AllFiles    ///< Mask of files to provide.
     );
       
     /**Move to the next file in the directory scan.
@@ -514,7 +543,7 @@ class PDirectory : public PFilePathString
     BOOL Next();
       
     /// Close the directory during or after a file list scan.
-    void Close();
+    virtual void Close();
 
     /**Get the name (without the volume or directory path) of the current
        entry in the directory scan. This may be the name of a file or a
@@ -530,7 +559,7 @@ class PDirectory : public PFilePathString
        @return
        string for directory entry.
      */
-    PFilePathString GetEntryName() const;
+    virtual PFilePathString GetEntryName() const;
 
     /**Determine if the directory entry currently being scanned is itself
        another directory entry.
@@ -541,15 +570,15 @@ class PDirectory : public PFilePathString
        @return
        TRUE if entry is a subdirectory.
      */
-    BOOL IsSubDir() const;
+    virtual BOOL IsSubDir() const;
 
     /**Get file information on the current directory entry.
     
        @return
        TRUE if file information was successfully retrieved.
      */
-    BOOL GetInfo(
-      PFileInfo & info    /// Object to receive the file information.
+    virtual BOOL GetInfo(
+      PFileInfo & info    ///< Object to receive the file information.
     ) const;
   //@}
 
@@ -557,17 +586,22 @@ class PDirectory : public PFilePathString
   protected:
     // New functions for class
     void Construct();
-      // Common constructor code
-
+    void Destruct()
+    { Close(); PFilePathString::Destruct(); }
 
     // Member variables
     /// Mask of file types that the directory scan will return.
     int scanMask;
 
-
 // Include platform dependent part of class
-#include <ptlib/pdirect.h>
+#ifdef _WIN32
+#include "msos/ptlib/pdirect.h"
+#else
+#include "unix/ptlib/pdirect.h"
+#endif
+
 };
 
+#endif
 
 // End Of File ///////////////////////////////////////////////////////////////
