@@ -25,6 +25,14 @@
  *                 Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
  * $Log: vidinput_v4l.cxx,v $
+ * Revision 1.15.2.8  2007/06/02 12:10:03  dsandras
+ * Added patch from Michael Smith <msmith cbnco com> refreshing video
+ * capabilities when appropriate. Thanks!
+ *
+ * Revision 1.15.2.7  2007/05/01 20:14:48  dsandras
+ * Fixed possible crash when opening V4L devices with 352x288 fixed
+ * width capability thanks Luc Saillard. (Ekiga #434223).
+ *
  * Revision 1.15.2.6  2007/04/10 21:17:56  dsandras
  * Added MJPEG support. Added workarounds for broken qspca driver.
  * Thanks to Luc Saillard (luc saillard org).
@@ -744,8 +752,7 @@ BOOL PVideoInputDevice_V4L::Open(const PString & devName, BOOL startImmediate)
   }
   
   // get the device capabilities
-  if (::ioctl(videoFd, VIDIOCGCAP, &videoCapability) < 0)  {
-    PTRACE(1,"PVideoInputDevice_V4L:: get device capablilities failed : "<< ::strerror(errno));
+  if (!RefreshCapabilities()) {
     ::close (videoFd);
     videoFd = -1;
     return FALSE;
@@ -809,7 +816,7 @@ BOOL PVideoInputDevice_V4L::Open(const PString & devName, BOOL startImmediate)
 	      BOOL false_positive = FALSE;
 	      unsigned int idx;
 	      for (idx = 0; idx < PARRAYSIZE(sensors_with_352x288_fixed_width); idx++) {
-		 if (strcmp(sensors_with_352x288_fixed_width[tbl], videoCapability.name) == 0) {
+		 if (strcmp(sensors_with_352x288_fixed_width[idx], videoCapability.name) == 0) {
 		    false_positive = TRUE;
 		    break;
 		 }
@@ -937,8 +944,11 @@ BOOL PVideoInputDevice_V4L::SetVideoFormat(VideoFormat newFormat)
   channel.norm = fmt[newFormat];
 
   // set the information
-  if (::ioctl(videoFd, VIDIOCSCHAN, &channel) >= 0)
+  if (::ioctl(videoFd, VIDIOCSCHAN, &channel) >= 0) {
+    // format change might affect frame size limits; grab them again
+    RefreshCapabilities();
     return TRUE;
+  }
 
   PTRACE(1,"VideoInputDevice SetChannel failed : "<< ::strerror(errno));  
 
@@ -988,6 +998,9 @@ BOOL PVideoInputDevice_V4L::SetChannel(int newChannel)
     return FALSE;
   }
   
+  // it's unlikely that channel change would affect frame size limits,
+  // but grab them again all the same
+  RefreshCapabilities();
   return TRUE;
 }
 
@@ -1026,6 +1039,8 @@ BOOL PVideoInputDevice_V4L::SetVideoChannelFormat (int newNumber, VideoFormat vi
     return FALSE;
   }
 
+  // format change may affect frame size limits
+  RefreshCapabilities();
   return TRUE;
 }
 
@@ -1612,4 +1627,14 @@ BOOL PVideoInputDevice_V4L::TestAllFormats()
   return TRUE;
 }
 
+BOOL PVideoInputDevice_V4L::RefreshCapabilities()
+{
+  if (::ioctl(videoFd, VIDIOCGCAP, &videoCapability) < 0)  {
+    PTRACE(1,"PVideoInputV4lDevice:: get device capablilities failed : "<< ::strerror(errno));
+    return FALSE;
+  }
+  return TRUE;
+}
+
+
 // End Of File ///////////////////////////////////////////////////////////////
