@@ -27,6 +27,155 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: tlibthrd.cxx,v $
+ * Revision 1.150.2.2  2006/03/01 08:34:33  csoutheren
+ * Backported patch #1439578 from CVS head
+ *
+ * Revision 1.150.2.1  2006/01/29 23:34:52  csoutheren
+ * Backported thread destruction patch from Derek Smithies
+ *
+ * Revision 1.152  2006/03/01 08:29:33  csoutheren
+ * Applied patch #1439578 PTrace / PThread::PX_ThreadEnd deadlock fix
+ * Thanks to Hannes Friederich
+ *
+ * Revision 1.151  2006/01/29 22:35:47  csoutheren
+ * Added fix for thread termination problems on SMP machines
+ * Thanks to Derek Smithies
+ *
+ * Revision 1.150  2005/12/05 22:35:24  csoutheren
+ * Only assert in PTimedMutex destructor if _DEBUG is enabled
+ *
+ * Revision 1.149  2005/12/04 22:07:26  csoutheren
+ * Fixed uninitialised variable
+ *
+ * Revision 1.148  2005/12/01 00:55:19  csoutheren
+ * Removed chance of endless loop in PTimedMutex destructor
+ *
+ * Revision 1.147  2005/11/25 00:06:12  csoutheren
+ * Applied patch #1364593 from Hannes Friederich
+ * Also changed so PTimesMutex is no longer descended from PSemaphore on
+ * non-Windows platforms
+ *
+ * Revision 1.146  2005/11/22 22:38:36  dsandras
+ * Removed Assert that was causing problem if the mutex is locked when being
+ * destroyed.
+ *
+ * Revision 1.145  2005/11/18 22:26:07  dsandras
+ * Removed a few more CONST's to match with previous commit and fix permanent
+ * deadlock.
+ *
+ * Revision 1.144  2005/11/14 22:41:53  csoutheren
+ * Reverted Wait and Signal to non-const - there is no way we can guarantee that all
+ * descendant classes everywhere will be changed over, so we have to keep the
+ * original API
+ *
+ * Revision 1.143  2005/11/04 09:44:30  csoutheren
+ * Applied patch #1324589
+ * Removed race conditions in PSemaphore and thread handling
+ * Thanks to Frederic Heem
+ *
+ * Revision 1.142  2005/11/04 06:56:10  csoutheren
+ * Added new class PSync as abstract base class for all mutex/sempahore classes
+ * Changed PCriticalSection to use Wait/Signal rather than Enter/Leave
+ * Changed Wait/Signal to be const member functions
+ * Renamed PMutex to PTimedMutex and made PMutex synonym for PCriticalSection.
+ * This allows use of very efficient mutex primitives in 99% of cases where timed waits
+ * are not needed
+ *
+ * Revision 1.141  2005/07/22 04:19:18  csoutheren
+ * Removed redundant check of thread ID introduced in last patch
+ * Removed race condition in thread shutdown found by Derek Smithies
+ *
+ * Revision 1.140  2005/07/21 13:04:11  csoutheren
+ * Removed race condition where activeThreads list does not contain
+ * thread until some time after thread is started. Fixed by moving
+ * list insertion to immediately after pthread_create, and using lock
+ *
+ * Revision 1.139  2005/07/21 00:09:08  csoutheren
+ * Added workaround for braindead behaviour of pthread_kill
+ * Thanks to "martin martin" <acevedoma@hotmail.com>
+ *
+ * Revision 1.138  2005/05/03 11:58:46  csoutheren
+ * Fixed various problems reported by valgrind
+ * Thanks to Paul Cadach
+ *
+ * Revision 1.137  2005/01/21 21:25:19  csoutheren
+ * Removed incorrect return in PThread::WaitForTermination
+ *
+ * Revision 1.136  2005/01/16 23:00:37  csoutheren
+ * Fixed problem when calling WaitForTermination from within the same thread
+ *
+ * Revision 1.135  2004/12/21 06:30:55  csoutheren
+ * Added explicit stack size for pthreads to minimise VM usage, thanks to Diana Cionoiu
+ *
+ * Revision 1.134  2004/09/02 07:55:44  csoutheren
+ * Added extra PXAbortBlock to WaitForTermination to assist in terminaing
+ * threads under certain conditions
+ *
+ * Revision 1.133  2004/06/24 11:29:44  csoutheren
+ * Changed to use pthread_mutex_timedlock for more efficient mutex wait operations
+ * Thanks to Michal Zygmuntowicz
+ *
+ * Revision 1.132  2004/06/01 07:42:20  csoutheren
+ * Restored memory allocation checking
+ * Added configure flag to enable, thanks to Derek Smithies
+ *
+ * Revision 1.131  2004/05/21 00:49:16  csoutheren
+ * Added PreShutdown to ~PProcess
+ *
+ * Revision 1.130  2004/04/27 04:36:47  rjongbloed
+ * Fixed occassional crash on exit due to level 5 trace in PProcess
+ *   destructor that needs an undestructed PProcess.
+ * Added some more logging for unblocking threads.
+ *
+ * Revision 1.129  2004/04/12 03:35:27  csoutheren
+ * Fixed problems with non-recursuve mutexes and critical sections on
+ * older compilers and libc
+ *
+ * Revision 1.128  2004/04/12 00:58:45  csoutheren
+ * Fixed PAtomicInteger on Linux, and modified PMutex to use it
+ *
+ * Revision 1.127  2004/04/11 07:58:08  csoutheren
+ * Added configure.in check for recursive mutexes, and changed implementation
+ * without recursive mutexes to use PCriticalSection or atomic word structs
+ *
+ * Revision 1.126  2004/03/24 02:37:04  csoutheren
+ * Fixed problem with incorrect usage of sem_timedwait
+ *
+ * Revision 1.125  2004/03/23 04:56:23  csoutheren
+ * Added patches to use XPG6 threading under Linux if available
+ * Thanks to Matthew Hodgson
+ *
+ * Revision 1.124  2004/02/01 11:23:16  dsandras
+ * Reverted previous Change and removed Yield call from Current (). Fix from Christian Meder <chris@onestepahead.de>. Thanks for your help, Christian!
+ *
+ * Revision 1.123  2004/01/31 13:49:18  dominance
+ * Added 2.6 performance fix as proposed by Christian Meder <chris@onestepahead.de>.
+ *
+ * Revision 1.122  2003/09/17 09:02:15  csoutheren
+ * Removed memory leak detection code
+ *
+ * Revision 1.121  2003/05/16 17:40:55  shawn
+ * On Mac OS X, thread with the highest priority should use fixed priority
+ * scheduling policy.  This avoids starvation caused by desktop activity.
+ *
+ * Revision 1.120  2003/05/02 00:58:40  dereks
+ * Add test for linux at the end of PMutex::Signal. Thanks Robert!
+ *
+ * Revision 1.119  2003/05/02 00:39:11  dereks
+ * Changes to make threading work on Redhat 9
+ *
+ * Revision 1.118  2003/04/24 12:03:13  rogerh
+ * Calling pthread_mutex_unlock() on a mutex which is not locked can be
+ * considered an error. NetBSD now enforce this error so we need to quickly
+ * try locking the mutex before unlocking it in ~PThread and ~PSemaphore.
+ *
+ * Revision 1.117  2003/04/08 03:29:31  robertj
+ * Fixed IsSuspeneded() so returns TRUE if thread not started yet, this makes
+ *   it the same as the Win32 semantics.
+ *
+ * Revision 1.116  2003/03/10 15:37:00  rogerh
+ * fix IsTerminated() function.
+ *
  * Revision 1.115  2003/03/07 00:07:15  robertj
  * Fixed Mac OS X patch which broke every other platform.
  *
@@ -412,20 +561,34 @@
  */
 
 #include <ptlib/socket.h>
-#include <sched.h>	// for sched_yield
+#include <sched.h>  // for sched_yield
 #include <pthread.h>
 #include <sys/resource.h>
 
 #ifdef P_RTEMS
-#define	SUSPEND_SIG SIGALRM
+#define SUSPEND_SIG SIGALRM
 #include <sched.h>
 #else
-#define	SUSPEND_SIG SIGVTALRM
+#define SUSPEND_SIG SIGVTALRM
 #endif
 
+#ifdef P_MACOSX
+#include <mach/mach.h>
+#include <mach/thread_policy.h>
+#include <sys/param.h>
+#include <sys/sysctl.h>
+// going to need the main thread for adjusting relative priority
+static pthread_t baseThread;
+#endif
+
+#ifdef P_HAS_SEMAPHORES_XPG6
+#include "semaphore.h"
+#endif
 
 int PX_NewHandle(const char *, int);
 
+#define PPThreadKill(id, sig)  PProcess::Current().PThreadKill(id, sig)
+
 
 #define PAssertPTHREAD(func, args) \
   { \
@@ -476,7 +639,6 @@ PDECLARE_CLASS(PHouseKeepingThread, PThread)
 
 
 static pthread_mutex_t MutexInitialiser = PTHREAD_MUTEX_INITIALIZER;
-static pthread_cond_t  CondInitialiser  = PTHREAD_COND_INITIALIZER;
 
 
 #define new PNEW
@@ -515,7 +677,7 @@ void PProcess::SignalTimerChange()
 #endif
   }
 
-  BYTE ch;
+  static BYTE ch = 0;
   write(timerChangePipe[1], &ch, 1);
 }
 
@@ -538,6 +700,11 @@ void PProcess::Construct()
   // initialise the housekeeping thread
   housekeepingThread = NULL;
 
+#ifdef P_MACOSX
+    // records the main thread for priority adjusting
+    baseThread = pthread_self();
+#endif
+
   CommonConstruct();
 }
 
@@ -569,6 +736,8 @@ BOOL PProcess::SetMaxHandles(int newMax)
 
 PProcess::~PProcess()
 {
+  PreShutdown();
+
   // Don't wait for housekeeper to stop if Terminate() is called from it.
   if (housekeepingThread != NULL && PThread::Current() != housekeepingThread) {
     housekeepingThread->SetClosing();
@@ -577,6 +746,18 @@ PProcess::~PProcess()
     delete housekeepingThread;
   }
   CommonDestruct();
+
+  PTRACE(5, "PWLib\tDestroyed process " << this);
+}
+
+BOOL PProcess::PThreadKill(pthread_t id, unsigned sig)
+{
+  PWaitAndSignal m(threadMutex);
+
+  if (!activeThreads.Contains((unsigned)id)) 
+    return FALSE;
+
+  return pthread_kill(id, sig) == 0;
 }
 
 
@@ -613,6 +794,8 @@ void PThread::InitialiseProcessThread()
   ((PProcess *)this)->activeThreads.DisallowDeleteObjects();
   ((PProcess *)this)->activeThreads.SetAt((unsigned)PX_threadId, this);
 
+  PX_firstTimeStart = FALSE;
+
   traceBlockIndentLevel = 0;
 }
 
@@ -666,10 +849,13 @@ PThread::~PThread()
   pthread_mutex_destroy(&PX_WaitSemMutex);
 #endif
 
+  // If the mutex was not locked, the unlock will fail */
+  pthread_mutex_trylock(&PX_suspendMutex);
   pthread_mutex_unlock(&PX_suspendMutex);
   pthread_mutex_destroy(&PX_suspendMutex);
 
-  PTRACE(5, "PWLib\tDestroyed thread " << this << ' ' << threadName);
+  if (this != &PProcess::Current())
+    PTRACE(5, "PWLib\tDestroyed thread " << this << ' ' << threadName);
 }
 
 
@@ -683,6 +869,10 @@ void PThread::Restart()
   pthread_attr_setdetachstate(&threadAttr, PTHREAD_CREATE_DETACHED);
 
 #if defined(P_LINUX)
+
+  // Set a decent (256K) stack size that won't eat all virtual memory
+  pthread_attr_setstacksize(&threadAttr, 16*PTHREAD_STACK_MIN);
+
   /*
     Set realtime scheduling if our effective user id is root (only then is this
     allowed) AND our priority is Highest.
@@ -705,7 +895,32 @@ void PThread::Restart()
   pthread_attr_setschedparam(&threadAttr, &sched_param);
 #endif
 
+  PProcess & process = PProcess::Current();
+  PINDEX newHighWaterMark = 0;
+  static PINDEX highWaterMark = 0;
+
+  // lock the thread list
+  process.threadMutex.Wait();
+
+  // create the thread
   PAssertPTHREAD(pthread_create, (&PX_threadId, &threadAttr, PX_ThreadStart, this));
+
+  // put the thread into the thread list
+  process.activeThreads.SetAt((unsigned)PX_threadId, this);
+  if (process.activeThreads.GetSize() > highWaterMark)
+    newHighWaterMark = highWaterMark = process.activeThreads.GetSize();
+
+  // unlock the thread list
+  process.threadMutex.Signal();
+
+  PTRACE_IF(4, newHighWaterMark > 0, "PWLib\tThread high water mark set: " << newHighWaterMark);
+
+#ifdef P_MACOSX
+  if (PX_priority == HighestPriority) {
+    PTRACE(1, "set thread to have the highest priority (MACOSX)");
+    SetPriority(HighestPriority);
+  }
+#endif
 }
 
 
@@ -747,11 +962,11 @@ void PThread::Suspend(BOOL susp)
     return;
   }
 
-#if defined(P_MAXOSX) && (P_MACOSX <= 55)
+#if defined(P_MACOSX) && (P_MACOSX <= 55)
   // Suspend - warn the user with an Assertion
   PAssertAlways("Cannot suspend threads on Mac OS X due to lack of pthread_kill()");
 #else
-  if (pthread_kill(PX_threadId, 0) == 0) {
+  if (PPThreadKill(PX_threadId, 0)) {
 
     // if suspending, then see if already suspended
     if (susp) {
@@ -759,7 +974,7 @@ void PThread::Suspend(BOOL susp)
       if (PX_suspendCount == 1) {
         if (PX_threadId != pthread_self()) {
           signal(SUSPEND_SIG, PX_SuspendSignalHandler);
-          pthread_kill(PX_threadId, SUSPEND_SIG);
+          PPThreadKill(PX_threadId, SUSPEND_SIG);
         }
         else {
           PAssertPTHREAD(pthread_mutex_unlock, (&PX_suspendMutex));
@@ -790,6 +1005,9 @@ void PThread::Resume()
 
 BOOL PThread::IsSuspended() const
 {
+  if (PX_firstTimeStart)
+    return TRUE;
+
   if (IsTerminated())
     return FALSE;
 
@@ -806,6 +1024,54 @@ void PThread::SetAutoDelete(AutoDeleteFlag deletion)
   autoDelete = deletion == AutoDeleteThread;
 }
 
+#ifdef P_MACOSX
+// obtain thread priority of the main thread
+static unsigned long
+GetThreadBasePriority ()
+{
+    thread_basic_info_data_t threadInfo;
+    policy_info_data_t       thePolicyInfo;
+    unsigned int             count;
+
+    if (baseThread == 0) {
+      return 0;
+    }
+
+    // get basic info
+    count = THREAD_BASIC_INFO_COUNT;
+    thread_info (pthread_mach_thread_np (baseThread), THREAD_BASIC_INFO,
+                 (integer_t*)&threadInfo, &count);
+
+    switch (threadInfo.policy) {
+    case POLICY_TIMESHARE:
+      count = POLICY_TIMESHARE_INFO_COUNT;
+      thread_info(pthread_mach_thread_np (baseThread),
+                  THREAD_SCHED_TIMESHARE_INFO,
+                  (integer_t*)&(thePolicyInfo.ts), &count);
+      return thePolicyInfo.ts.base_priority;
+
+    case POLICY_FIFO:
+      count = POLICY_FIFO_INFO_COUNT;
+      thread_info(pthread_mach_thread_np (baseThread),
+                  THREAD_SCHED_FIFO_INFO,
+                  (integer_t*)&(thePolicyInfo.fifo), &count);
+      if (thePolicyInfo.fifo.depressed) 
+        return thePolicyInfo.fifo.depress_priority;
+      return thePolicyInfo.fifo.base_priority;
+
+    case POLICY_RR:
+      count = POLICY_RR_INFO_COUNT;
+      thread_info(pthread_mach_thread_np (baseThread),
+                  THREAD_SCHED_RR_INFO,
+                  (integer_t*)&(thePolicyInfo.rr), &count);
+      if (thePolicyInfo.rr.depressed) 
+        return thePolicyInfo.rr.depress_priority;
+      return thePolicyInfo.rr.base_priority;
+    }
+
+    return 0;
+}
+#endif
 
 void PThread::SetPriority(Priority priorityLevel)
 {
@@ -829,6 +1095,48 @@ void PThread::SetPriority(Priority priorityLevel)
     PAssertPTHREAD(pthread_setschedparam, (PX_threadId, SCHED_OTHER, &sched_param));
   }
 #endif
+
+#if defined(P_MACOSX)
+  if (IsTerminated())
+    return;
+
+  if (priorityLevel == HighestPriority) {
+    /* get fixed priority */
+    {
+      int result;
+
+      thread_extended_policy_data_t   theFixedPolicy;
+      thread_precedence_policy_data_t thePrecedencePolicy;
+      long                            relativePriority;
+
+      theFixedPolicy.timeshare = false; // set to true for a non-fixed thread
+      result = thread_policy_set (pthread_mach_thread_np(PX_threadId),
+                                  THREAD_EXTENDED_POLICY,
+                                  (thread_policy_t)&theFixedPolicy,
+                                  THREAD_EXTENDED_POLICY_COUNT);
+      if (result != KERN_SUCCESS) {
+        PTRACE(1, "thread_policy - Couldn't set thread as fixed priority.");
+      }
+
+      // set priority
+
+      // precedency policy's "importance" value is relative to
+      // spawning thread's priority
+      
+      relativePriority = 62 - GetThreadBasePriority();
+      PTRACE(1,  "relativePriority is " << relativePriority << " base priority is " << GetThreadBasePriority());
+      
+      thePrecedencePolicy.importance = relativePriority;
+      result = thread_policy_set (pthread_mach_thread_np(PX_threadId),
+                                  THREAD_PRECEDENCE_POLICY,
+                                  (thread_policy_t)&thePrecedencePolicy, 
+                                  THREAD_PRECEDENCE_POLICY_COUNT);
+      if (result != KERN_SUCCESS) {
+        PTRACE(1, "thread_policy - Couldn't set thread priority.");
+      }
+    }
+  }
+#endif
 }
 
 
@@ -947,7 +1255,7 @@ void PThread::Terminate()
   PXAbortBlock();
   WaitForTermination(20);
 
-#ifndef P_HAS_SEMAPHORES
+#if !defined(P_HAS_SEMAPHORES) && !defined(P_HAS_NAMED_SEMAPHORES)
   PAssertPTHREAD(pthread_mutex_lock, (&PX_WaitSemMutex));
   if (PX_waitingSemaphore != NULL) {
     PAssertPTHREAD(pthread_mutex_lock, (&PX_waitingSemaphore->mutex));
@@ -959,34 +1267,31 @@ void PThread::Terminate()
 #endif
 
 #if ( defined(P_NETBSD) && defined(P_NO_CANCEL) )
-  pthread_kill(PX_threadId,SIGKILL);
+  PPThreadKill(PX_threadId,SIGKILL);
 #else
-  pthread_cancel(PX_threadId);
+  if (PX_threadId) {
+    pthread_cancel(PX_threadId);
+  }
 #endif
 }
 
 
 BOOL PThread::IsTerminated() const
 {
-  if (PX_threadId == 0)
-    return TRUE;
-
-#if defined(P_MAXOSX) && (P_MACOSX <= 55)
-  // MacOS X does not support pthread_kill so we cannot use it
-  // to test the validity of the thread
-  if (pthread_kill(PX_threadId, 0) != 0)
-    return TRUE;
-#endif
-
-  PTRACE(7, "PWLib\tIsTerminated(" << (void *)this << ") not dead yet");
-  return FALSE;
+  pthread_t id = PX_threadId;
+  return (id == 0) || !PPThreadKill(id, 0);
 }
 
 
 void PThread::WaitForTermination() const
 {
-  PAssert(Current() != this, "Waiting for self termination!");
+  if (this == Current()) {
+    PTRACE(2, "WaitForTermination short circuited");
+    return;
+  }
   
+  PXAbortBlock();   // this assist in clean shutdowns on some systems
+
   while (!IsTerminated()) {
     Sleep(10); // sleep for 10ms. This slows down the busy loop removing 100%
                // CPU usage and also yeilds so other threads can run.
@@ -996,10 +1301,14 @@ void PThread::WaitForTermination() const
 
 BOOL PThread::WaitForTermination(const PTimeInterval & maxWait) const
 {
-  PAssert(Current() != this, "Waiting for self termination!");
+  if (this == Current()) {
+    PTRACE(2, "WaitForTermination(t) short circuited");
+    return TRUE;
+  }
   
   PTRACE(6, "PWLib\tWaitForTermination(" << maxWait << ')');
 
+  PXAbortBlock();   // this assist in clean shutdowns on some systems
   PTimer timeout = maxWait;
   while (!IsTerminated()) {
     if (timeout == 0)
@@ -1013,36 +1322,15 @@ BOOL PThread::WaitForTermination(const PTimeInterval & maxWait) const
 
 void * PThread::PX_ThreadStart(void * arg)
 { 
-  pthread_t threadId = pthread_self();
-
-  // self-detach
-  pthread_detach(threadId);
-
   PThread * thread = (PThread *)arg;
-
+  //don't need to detach the the thread, it was created in the PTHREAD_CREATE_DETACHED state
   // Added this to guarantee that the thread creation (PThread::Restart)
   // has completed before we start the thread. Then the PX_threadId has
   // been set.
   pthread_mutex_lock(&thread->PX_suspendMutex);
-
   thread->SetThreadName(thread->GetThreadName());
-
   pthread_mutex_unlock(&thread->PX_suspendMutex);
 
-  PProcess & process = PProcess::Current();
-
-  PINDEX newHighWaterMark = 0;
-  static PINDEX highWaterMark = 0;
-
-  // add thread to thread list
-  process.threadMutex.Wait();
-  process.activeThreads.SetAt((unsigned)threadId, thread);
-  if (process.activeThreads.GetSize() > highWaterMark)
-    newHighWaterMark = highWaterMark = process.activeThreads.GetSize();
-  process.threadMutex.Signal();
-
-  PTRACE_IF(4, newHighWaterMark > 0, "PWLib\tThread high water mark set: " << newHighWaterMark);
-
   // make sure the cleanup routine is called when the thread exits
   pthread_cleanup_push(PThread::PX_ThreadEnd, arg);
 
@@ -1060,36 +1348,41 @@ void * PThread::PX_ThreadStart(void * arg)
 
 void PThread::PX_ThreadEnd(void * arg)
 {
+  PProcess & process = PProcess::Current();
+  process.threadMutex.Wait();
+
   PThread * thread = (PThread *)arg;
   pthread_t id = thread->GetThreadId();
   if (id == 0) {
     // Don't know why, but pthreads under Linux at least can call this function
     // multiple times! Probably a bug, but we have to allow for it.
+    process.threadMutex.Signal();
     PTRACE(2, "PWLib\tAttempted to multiply end thread " << thread << " ThreadID=" << (void *)id);
     return;
-  }
+  }  
 
-  PTRACE(5, "PWLib\tEnded thread " << thread << ' ' << thread->threadName);
-
-  // remove this thread from the active thread list
-  PProcess & process = PProcess::Current();
-  process.threadMutex.Wait();
+ // remove this thread from the active thread list
   process.activeThreads.SetAt((unsigned)id, NULL);
-  process.threadMutex.Signal();
 
   // delete the thread if required, note this is done this way to avoid
   // a race condition, the thread ID cannot be zeroed before the if!
   if (thread->autoDelete) {
     thread->PX_threadId = 0;  // Prevent terminating terminated thread
+    process.threadMutex.Signal();
+    PTRACE(5, "PWLib\tEnded thread " << thread << ' ' << thread->threadName);
 
-    // Now should be safe to delete the thread!
+    /* It is now safe to delete this thread. Note that this thread
+       is deleted after the process.threadMutex.Signal(), which means
+       PWaitAndSignal(process.threadMutex) could not be used */
     delete thread;
   }
-  else
+  else {
     thread->PX_threadId = 0;
+    process.threadMutex.Signal();
+    PTRACE(5, "PWLib\tEnded thread " << thread << ' ' << thread->threadName);
+  }
 }
 
-
 int PThread::PXBlockOnIO(int handle, int type, const PTimeInterval & timeout)
 {
   PTRACE(7, "PWLib\tPThread::PXBlockOnIO(" << handle << ',' << type << ')');
@@ -1142,7 +1435,7 @@ int PThread::PXBlockOnIO(int handle, int type, const PTimeInterval & timeout)
     ::read(unblockPipe[0], &ch, 1);
     errno = EINTR;
     retval =  -1;
-    PTRACE(6, "PWLib\tUnblocked I/O");
+    PTRACE(6, "PWLib\tUnblocked I/O fd=" << unblockPipe[0]);
   }
 
   return retval;
@@ -1150,8 +1443,9 @@ int PThread::PXBlockOnIO(int handle, int type, const PTimeInterval & timeout)
 
 void PThread::PXAbortBlock() const
 {
-  BYTE ch;
+  static BYTE ch = 0;
   ::write(unblockPipe[1], &ch, 1);
+  PTRACE(6, "PWLib\tUnblocking I/O fd=" << unblockPipe[0] << " thread=" << GetThreadName());
 }
 
 
@@ -1160,28 +1454,44 @@ void PThread::PXAbortBlock() const
 PSemaphore::PSemaphore(PXClass pxc)
 {
   pxClass = pxc;
-  mutex   = MutexInitialiser;
-  condVar = CondInitialiser;
 
   // these should never be used, as this constructor is
   // only used for PMutex and PSyncPoint and they have their
   // own copy constructors
+  
   initialVar = maxCountVar = 0;
+  
+  if(pxClass == PXSemaphore) {
+#if defined(P_HAS_SEMAPHORES)
+    /* call sem_init, otherwise sem_destroy fails*/
+    PAssertPTHREAD(sem_init, (&semId, 0, 0));
+#elif defined(P_HAS_NAMED_SEMAPHORES)
+    semId = CreateSem(0);
+#else
+    currentCount = maximumCount = 0;
+    queuedLocks = 0;
+    pthread_mutex_init(&mutex, NULL);
+    pthread_cond_init(&condVar, NULL);
+#endif
+  }
 }
 
 
 PSemaphore::PSemaphore(unsigned initial, unsigned maxCount)
 {
   pxClass = PXSemaphore;
-  mutex   = MutexInitialiser;
-  condVar = CondInitialiser;
 
   initialVar  = initial;
   maxCountVar = maxCount;
 
-#ifdef P_HAS_SEMAPHORES
+#if defined(P_HAS_SEMAPHORES)
   PAssertPTHREAD(sem_init, (&semId, 0, initial));
+#elif defined(P_HAS_NAMED_SEMAPHORES)
+  semId = CreateSem(initialVar);
 #else
+  PAssertPTHREAD(pthread_mutex_init, (&mutex, NULL));
+  PAssertPTHREAD(pthread_cond_init, (&condVar, NULL));
+  
   PAssert(maxCount > 0, "Invalid semaphore maximum.");
   if (initial > maxCount)
     initial = maxCount;
@@ -1192,48 +1502,77 @@ PSemaphore::PSemaphore(unsigned initial, unsigned maxCount)
 #endif
 }
 
-PSemaphore::PSemaphore(const PSemaphore & sem)
+
+PSemaphore::PSemaphore(const PSemaphore & sem) 
 {
   pxClass = sem.GetSemClass();
-  mutex   = MutexInitialiser;
-  condVar = CondInitialiser;
 
   initialVar  = sem.GetInitial();
   maxCountVar = sem.GetMaxCount();
 
-#ifdef P_HAS_SEMAPHORES
-  PAssertPTHREAD(sem_init, (&semId, 0, initialVar));
+  if(pxClass == PXSemaphore) {
+#if defined(P_HAS_SEMAPHORES)
+    PAssertPTHREAD(sem_init, (&semId, 0, initialVar));
+#elif defined(P_HAS_NAMED_SEMAPHORES)
+    semId = CreateSem(initialVar);
 #else
-  PAssert(maxCountVar > 0, "Invalid semaphore maximum.");
-  if (initialVar > maxCountVar)
-    initialVar = maxCountVar;
+    PAssertPTHREAD(pthread_mutex_init, (&mutex, NULL));
+    PAssertPTHREAD(pthread_cond_init, (&condVar, NULL));
+  
+    PAssert(maxCountVar > 0, "Invalid semaphore maximum.");
+    if (initialVar > maxCountVar)
+      initialVar = maxCountVar;
 
-  currentCount = initialVar;
-  maximumCount = maxCountVar;
-  queuedLocks  = 0;
-#endif
-}
-
-PSemaphore::~PSemaphore()
-{
-  pthread_cond_destroy(&condVar);
-  pthread_mutex_unlock(&mutex);
-  pthread_mutex_destroy(&mutex);
-
-  if (pxClass == PXSemaphore) {
-#ifdef P_HAS_SEMAPHORES
-    PAssertPTHREAD(sem_destroy, (&semId));
-#else
-    PAssert(queuedLocks == 0, "Semaphore destroyed with queued locks");
+    currentCount = initialVar;
+    maximumCount = maxCountVar;
+    queuedLocks  = 0;
 #endif
   }
 }
 
-
-void PSemaphore::Wait()
+PSemaphore::~PSemaphore()
 {
-#ifdef P_HAS_SEMAPHORES
+  if(pxClass == PXSemaphore) {
+#if defined(P_HAS_SEMAPHORES)
+    PAssertPTHREAD(sem_destroy, (&semId));
+#elif defined(P_HAS_NAMED_SEMAPHORES)
+    PAssertPTHREAD(sem_close, (semId));
+#else
+    PAssert(queuedLocks == 0, "Semaphore destroyed with queued locks");
+    PAssertPTHREAD(pthread_mutex_destroy, (&mutex));
+    PAssertPTHREAD(pthread_cond_destroy, (&condVar));
+#endif
+  }
+}
+
+#if defined(P_HAS_NAMED_SEMAPHORES)
+sem_t * PSemaphore::CreateSem(unsigned initialValue)
+{
+  sem_t *sem;
+
+  // Since sem_open and sem_unlink are two operations, there is a small
+  // window of opportunity that two simultaneous accesses may return
+  // the same semaphore. Therefore, the static mutex is used to
+  // prevent this, even if the chances are small
+  static pthread_mutex_t semCreationMutex = PTHREAD_MUTEX_INITIALIZER;
+  PAssertPTHREAD(pthread_mutex_lock, (&semCreationMutex));
+  
+  sem_unlink("/pwlib_sem");
+  sem = sem_open("/pwlib_sem", (O_CREAT | O_EXCL), 700, initialValue);
+  
+  PAssertPTHREAD(pthread_mutex_unlock, (&semCreationMutex));
+  
+  PAssert(((int)sem != SEM_FAILED), "Couldn't create named semaphore");
+  return sem;
+}
+#endif
+
+void PSemaphore::Wait() 
+{
+#if defined(P_HAS_SEMAPHORES)
   PAssertPTHREAD(sem_wait, (&semId));
+#elif defined(P_HAS_NAMED_SEMAPHORES)
+  PAssertPTHREAD(sem_wait, (semId));
 #else
   PAssertPTHREAD(pthread_mutex_lock, (&mutex));
 
@@ -1255,7 +1594,7 @@ void PSemaphore::Wait()
 }
 
 
-BOOL PSemaphore::Wait(const PTimeInterval & waitTime)
+BOOL PSemaphore::Wait(const PTimeInterval & waitTime) 
 {
   if (waitTime == PMaxTimeInterval) {
     Wait();
@@ -1266,8 +1605,23 @@ BOOL PSemaphore::Wait(const PTimeInterval & waitTime)
   PTime finishTime;
   finishTime += waitTime;
 
-#ifdef P_HAS_SEMAPHORES
+#if defined(P_HAS_SEMAPHORES)
+#ifdef P_HAS_SEMAPHORES_XPG6
+  // use proper timed spinlocks if supported.
+  // http://www.opengroup.org/onlinepubs/007904975/functions/sem_timedwait.html
 
+  struct timespec absTime;
+  absTime.tv_sec  = finishTime.GetTimeInSeconds();
+  absTime.tv_nsec = finishTime.GetMicrosecond() * 1000;
+
+  if (sem_timedwait(&semId, &absTime) == 0) {
+    return TRUE;
+  }
+  else {
+    return FALSE;
+  }
+
+#else
   // loop until timeout, or semaphore becomes available
   // don't use a PTimer, as this causes the housekeeping
   // thread to get very busy
@@ -1275,11 +1629,26 @@ BOOL PSemaphore::Wait(const PTimeInterval & waitTime)
     if (sem_trywait(&semId) == 0)
       return TRUE;
 
-    PThread::Yield(); // One time slice
+#if defined(P_LINUX)
+  // sched_yield in a tight loop is bad karma
+  // for the linux scheduler: http://www.ussg.iu.edu/hypermail/linux/kernel/0312.2/1127.html
+    PThread::Current()->Sleep(10);
+#else
+    PThread::Yield();
+#endif
   } while (PTime() < finishTime);
 
   return FALSE;
 
+#endif
+#elif defined(P_HAS_NAMED_SEMAPHORES)
+  do {
+    if(sem_trywait(semId) == 0)
+      return TRUE;
+    PThread::Current()->Sleep(10);
+  } while (PTime() < finishTime);
+  
+  return FALSE;
 #else
 
   struct timespec absTime;
@@ -1318,8 +1687,10 @@ BOOL PSemaphore::Wait(const PTimeInterval & waitTime)
 
 void PSemaphore::Signal()
 {
-#ifdef P_HAS_SEMAPHORES
+#if defined(P_HAS_SEMAPHORES)
   PAssertPTHREAD(sem_post, (&semId));
+#elif defined(P_HAS_NAMED_SEMAPHORES)
+  PAssertPTHREAD(sem_post, (semId));
 #else
   PAssertPTHREAD(pthread_mutex_lock, (&mutex));
 
@@ -1336,53 +1707,78 @@ void PSemaphore::Signal()
 
 BOOL PSemaphore::WillBlock() const
 {
-#ifdef P_HAS_SEMAPHORES
+#if defined(P_HAS_SEMAPHORES)
   if (sem_trywait((sem_t *)&semId) != 0) {
     PAssertOS(errno == EAGAIN || errno == EINTR);
     return TRUE;
   }
   PAssertPTHREAD(sem_post, ((sem_t *)&semId));
   return FALSE;
+#elif defined(P_HAS_NAMED_SEMAPHORES)
+  if (sem_trywait(semId) != 0) {
+    PAssertOS(errno == EAGAIN || errno == EINTR);
+    return TRUE;
+  }
+  PAssertPTHREAD(sem_post, (semId));
+  return FALSE;
 #else
   return currentCount == 0;
 #endif
 }
 
-#if defined(P_QNX) && defined(P_HAS_RECURSIVE_MUTEX)
+#if defined(P_QNX) && (P_HAS_RECURSIVE_MUTEX == 1)
+#define PTHREAD_MUTEX_RECURSIVE_NP PTHREAD_MUTEX_RECURSIVE
+#endif
+#if defined(P_MACOSX) && (P_HAS_RECURSIVE_MUTEX == 1)
 #define PTHREAD_MUTEX_RECURSIVE_NP PTHREAD_MUTEX_RECURSIVE
 #endif
 
-PMutex::PMutex()
-  : PSemaphore(PXMutex)
+PTimedMutex::PTimedMutex()
+//  : PSemaphore(PXMutex)
 {
-#ifdef P_HAS_RECURSIVE_MUTEX
+#if P_HAS_RECURSIVE_MUTEX
   pthread_mutexattr_t attr;
-  pthread_mutexattr_init(&attr);
-  pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE_NP);
-  pthread_mutex_init(&mutex, &attr);
+  PAssertPTHREAD(pthread_mutexattr_init, (&attr));
+  PAssertPTHREAD(pthread_mutexattr_settype, (&attr, PTHREAD_MUTEX_RECURSIVE_NP));
+  PAssertPTHREAD(pthread_mutex_init, (&mutex, &attr));
+  PAssertPTHREAD(pthread_mutexattr_destroy, (&attr));
 #else
+  PAssertPTHREAD(pthread_mutex_init, (&mutex, NULL));
   ownerThreadId = (pthread_t)-1;
-  lockCount = 0;
 #endif
 }
 
-PMutex::PMutex(const PMutex & /*mut*/)
-  : PSemaphore(PXMutex)
+PTimedMutex::PTimedMutex(const PTimedMutex & /*mut*/)
+//  : PSemaphore(PXMutex)
 {
-#ifdef P_HAS_RECURSIVE_MUTEX
+#if P_HAS_RECURSIVE_MUTEX
   pthread_mutexattr_t attr;
-  pthread_mutexattr_init(&attr);
-  pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE_NP);
-  pthread_mutex_init(&mutex, &attr);
+  PAssertPTHREAD(pthread_mutexattr_init, (&attr));
+  PAssertPTHREAD(pthread_mutexattr_settype, (&attr, PTHREAD_MUTEX_RECURSIVE_NP));
+  PAssertPTHREAD(pthread_mutex_init, (&mutex, &attr));
+  PAssertPTHREAD(pthread_mutexattr_destroy, (&attr));
 #else
+  pthread_mutex_init(&mutex, NULL);
   ownerThreadId = (pthread_t)-1;
-  lockCount = 0;
 #endif
 }
 
-void PMutex::Wait()
+PTimedMutex::~PTimedMutex()
 {
-#ifndef P_HAS_RECURSIVE_MUTEX
+  int result = pthread_mutex_destroy(&mutex);
+  PINDEX i = 0;
+  while ((result == EBUSY) && (i++ < 20)) {
+    pthread_mutex_unlock(&mutex);
+    result = pthread_mutex_destroy(&mutex);
+  }
+#ifdef _DEBUG
+  PAssert((result == 0), "Error destroying mutex");
+#endif
+}
+
+void PTimedMutex::Wait() 
+{
+#if P_HAS_RECURSIVE_MUTEX == 0
   pthread_t currentThreadId = pthread_self();
 
   // if the mutex is already acquired by this thread,
@@ -1390,7 +1786,7 @@ void PMutex::Wait()
   if (pthread_equal(ownerThreadId, currentThreadId)) {
     // Note this does not need a lock as it can only be touched by the thread
     // which already has the mutex locked.
-    lockCount++;
+    ++lockCount;
     return;
   }
 #endif
@@ -1398,8 +1794,8 @@ void PMutex::Wait()
   // acquire the lock for real
   PAssertPTHREAD(pthread_mutex_lock, (&mutex));
 
-#ifndef P_HAS_RECURSIVE_MUTEX
-  PAssert((ownerThreadId == (pthread_t)-1) && (lockCount == 0),
+#if P_HAS_RECURSIVE_MUTEX == 0
+  PAssert((ownerThreadId == (pthread_t)-1) && (lockCount.IsZero()),
           "PMutex acquired whilst locked by another thread");
   // Note this is protected by the mutex itself only the thread with
   // the lock can alter it.
@@ -1408,7 +1804,7 @@ void PMutex::Wait()
 }
 
 
-BOOL PMutex::Wait(const PTimeInterval & waitTime)
+BOOL PTimedMutex::Wait(const PTimeInterval & waitTime) 
 {
   // if waiting indefinitely, then do so
   if (waitTime == PMaxTimeInterval) {
@@ -1416,7 +1812,7 @@ BOOL PMutex::Wait(const PTimeInterval & waitTime)
     return TRUE;
   }
 
-#ifndef P_HAS_RECURSIVE_MUTEX
+#if P_HAS_RECURSIVE_MUTEX == 0
   // get the current thread ID
   pthread_t currentThreadId = pthread_self();
 
@@ -1424,7 +1820,7 @@ BOOL PMutex::Wait(const PTimeInterval & waitTime)
   if (pthread_equal(ownerThreadId, currentThreadId)) {
     // Note this does not need a lock as it can only be touched by the thread
     // which already has the mutex locked.
-    lockCount++;
+    ++lockCount;
     return TRUE;
   }
 #endif
@@ -1433,15 +1829,40 @@ BOOL PMutex::Wait(const PTimeInterval & waitTime)
   PTime finishTime;
   finishTime += waitTime;
 
+#if P_PTHREADS_XPG6
+  
+  struct timespec absTime;
+  absTime.tv_sec  = finishTime.GetTimeInSeconds();
+  absTime.tv_nsec = finishTime.GetMicrosecond() * 1000;
+
+#if P_HAS_RECURSIVE_MUTEX
+  return pthread_mutex_timedlock(&mutex, &absTime) == 0;
+#else
+
+  if (pthread_mutex_timedlock(&mutex, &absTime) != 0)
+    return FALSE;
+
+  PAssert((ownerThreadId == (pthread_t)-1) && (lockCount.IsZero()),
+          "PMutex acquired whilst locked by another thread");
+
+  // Note this is protected by the mutex itself only the thread with
+  // the lock can alter it.
+  ownerThreadId = currentThreadId;
+  return TRUE;
+
+#endif
+
+#else // P_PTHREADS_XPG6
+
   do {
     if (pthread_mutex_trylock(&mutex) == 0) {
-#ifndef P_HAS_RECURSIVE_MUTEX
-      PAssert((ownerThreadId == (pthread_t)-1) && (lockCount == 0),
+#if P_HAS_RECURSIVE_MUTEX == 0
+      PAssert((ownerThreadId == (pthread_t)-1) && (lockCount.IsZero()),
               "PMutex acquired whilst locked by another thread");
       // Note this is protected by the mutex itself only the thread with
       // the lock can alter it.
       ownerThreadId = currentThreadId;
-#endif
+#endif // P_HAS_RECURSIVE_MUTEX
 
       return TRUE;
     }
@@ -1450,12 +1871,14 @@ BOOL PMutex::Wait(const PTimeInterval & waitTime)
   } while (PTime() < finishTime);
 
   return FALSE;
+
+#endif // P_PTHREADS_XPG6
 }
 
 
-void PMutex::Signal()
+void PTimedMutex::Signal()
 {
-#ifndef P_HAS_RECURSIVE_MUTEX
+#if P_HAS_RECURSIVE_MUTEX == 0
   if (!pthread_equal(ownerThreadId, pthread_self())) {
     PAssertAlways("PMutex signal failed - no matching wait or signal by wrong thread");
     return;
@@ -1464,8 +1887,8 @@ void PMutex::Signal()
   // if lock was recursively acquired, then decrement the counter
   // Note this does not need a separate lock as it can only be touched by the thread
   // which already has the mutex locked.
-  if (lockCount > 0) {
-    lockCount--;
+  if (!lockCount.IsZero()) {
+    --lockCount;
     return;
   }
 
@@ -1477,9 +1900,9 @@ void PMutex::Signal()
 }
 
 
-BOOL PMutex::WillBlock() const
+BOOL PTimedMutex::WillBlock() const
 {
-#ifndef P_HAS_RECURSIVE_MUTEX
+#if P_HAS_RECURSIVE_MUTEX == 0
   pthread_t currentThreadId = pthread_self();
   if (currentThreadId == ownerThreadId)
     return FALSE;
@@ -1497,15 +1920,25 @@ BOOL PMutex::WillBlock() const
 PSyncPoint::PSyncPoint()
   : PSemaphore(PXSyncPoint)
 {
+  PAssertPTHREAD(pthread_mutex_init, (&mutex, NULL));
+  PAssertPTHREAD(pthread_cond_init, (&condVar, NULL));
   signalCount = 0;
 }
 
 PSyncPoint::PSyncPoint(const PSyncPoint &)
   : PSemaphore(PXSyncPoint)
 {
+  PAssertPTHREAD(pthread_mutex_init, (&mutex, NULL));
+  PAssertPTHREAD(pthread_cond_init, (&condVar, NULL));
   signalCount = 0;
 }
 
+PSyncPoint::~PSyncPoint()
+{
+  PAssertPTHREAD(pthread_mutex_destroy, (&mutex));
+  PAssertPTHREAD(pthread_cond_destroy, (&condVar));
+}
+
 void PSyncPoint::Wait()
 {
   PAssertPTHREAD(pthread_mutex_lock, (&mutex));
